Object.defineProperty(exports, '__esModule', { value: true });

var tslib = require('tslib');
var fs = require('fs');
var path = require('path');
var rollup = require('rollup');
var rollupPluginSwc3 = require('rollup-plugin-swc3');
var commonjs = require('@rollup/plugin-commonjs');
var shebang = require('rollup-plugin-preserve-shebang');
var json = require('@rollup/plugin-json');
var nodeResolve = require('@rollup/plugin-node-resolve');
require('arg');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var commonjs__default = /*#__PURE__*/_interopDefaultLegacy(commonjs);
var shebang__default = /*#__PURE__*/_interopDefaultLegacy(shebang);
var json__default = /*#__PURE__*/_interopDefaultLegacy(json);
var nodeResolve__default = /*#__PURE__*/_interopDefaultLegacy(nodeResolve);

var rootDir = process.cwd();
var config = {
    rootDir: rootDir
};

function getPackageMeta() {
    var pkgFilePath = path__default["default"].resolve(config.rootDir, "package.json");
    var targetPackageJson = {};
    try {
        targetPackageJson = JSON.parse(fs__default["default"].readFileSync(pkgFilePath, {
            encoding: "utf-8"
        }));
    } catch (_) {}
    return targetPackageJson;
}
var logger = {
    log: function log() {
        var args = [];
        for(var _i = 0; _i < arguments.length; _i++){
            args[_i] = arguments[_i];
        }
        console.log.apply(console, args);
    },
    warn: function warn() {
        var args = [];
        for(var _i = 0; _i < arguments.length; _i++){
            args[_i] = arguments[_i];
        }
        console.log.apply(console, tslib.__spreadArray(tslib.__spreadArray([
            "\x1b[33m"
        ], args, false), [
            "\x1b[0m"
        ], false));
    },
    error: function error() {
        var args = [];
        for(var _i = 0; _i < arguments.length; _i++){
            args[_i] = arguments[_i];
        }
        console.error.apply(console, tslib.__spreadArray(tslib.__spreadArray([
            "\x1b[31m"
        ], args, false), [
            "\x1b[0m"
        ], false));
    }
};

var Module = require("module").Module;
var minifyOptions = {
    compress: true,
    format: {
        comments: "some",
        wrapFuncArgs: false,
        preserveAnnotations: true
    },
    mangle: true
};
var hasLoggedTsWarning = false;
function resolveTypescript() {
    var ts;
    var m = new Module("", null);
    m.paths = Module._nodeModulePaths(config.rootDir);
    try {
        ts = m.require("typescript");
    } catch (_) {
        if (!hasLoggedTsWarning) {
            hasLoggedTsWarning = true;
            logger.error("Could not load TypeScript compiler. Try `yarn add --dev typescript`");
            process.exit(1);
        }
    }
    return ts;
}
function getDistPath(distPath) {
    return path.resolve(config.rootDir, distPath);
}
function createInputConfig(entry, pkg, options, _a) {
    var _b;
    var _c;
    var tsconfigPath = _a.tsconfigPath, tsCompilerOptions = _a.tsCompilerOptions;
    var externals = [
        pkg.peerDependencies,
        pkg.dependencies,
        pkg.peerDependenciesMeta
    ].filter(function(n) {
        return Boolean(n);
    }).map(function(o) {
        return Object.keys(o);
    }).reduce(function(a, b) {
        return a.concat(b);
    }, []).concat(((_c = options.external) !== null && _c !== void 0 ? _c : []).concat(pkg.name ? [
        pkg.name
    ] : []));
    var useTypescript = options.useTypescript, runtime = options.runtime, jscTarget = options.target, minify = options.minify, exportCondition = options.exportCondition;
    var typings = pkg.types || pkg.typings;
    var cwd = config.rootDir;
    var tsPath;
    var declarationDir;
    if (useTypescript) {
        var tsconfig = path.resolve(cwd, "tsconfig.json");
        tsPath = fs__default["default"].existsSync(tsconfig) ? tsconfig : undefined;
        if (typings) {
            declarationDir = path.dirname(path.resolve(cwd, typings));
        }
        if (exportCondition) {
            var exportConditionDistFolder = path.dirname(typeof exportCondition.export === "string" ? exportCondition.export : Object.values(exportCondition.export)[0]);
            declarationDir = path.resolve(config.rootDir, exportConditionDistFolder);
        }
    }
    var isMainExport = !exportCondition || exportCondition.name === ".";
    var plugins = [
        nodeResolve__default["default"]({
            preferBuiltins: runtime === "node",
            extensions: [
                ".mjs",
                ".js",
                ".json",
                ".node",
                ".jsx"
            ]
        }),
        commonjs__default["default"]({
            include: /node_modules\//
        }),
        json__default["default"](),
        shebang__default["default"](),
        useTypescript && isMainExport && require("@rollup/plugin-typescript")(tslib.__assign({
            tsconfig: tsPath,
            typescript: resolveTypescript(),
            jsx: (tsCompilerOptions === null || tsCompilerOptions === void 0 ? void 0 : tsCompilerOptions.jsx) || "react",
            module: "ES6",
            target: jscTarget,
            noEmitOnError: !options.watch,
            sourceMap: options.sourcemap,
            declaration: !!typings,
            emitDeclarationOnly: true
        }, !!typings && {
            declarationDir: declarationDir
        })),
        rollupPluginSwc3.swc({
            include: /\.(m|c)?[jt]sx?$/,
            exclude: "node_modules",
            tsconfig: tsconfigPath,
            jsc: tslib.__assign({
                target: jscTarget,
                loose: true,
                externalHelpers: false,
                parser: (_b = {
                    syntax: useTypescript ? "typescript" : "ecmascript"
                }, _b[useTypescript ? "tsx" : "jsx"] = true, _b.privateMethod = true, _b.classPrivateProperty = true, _b.exportDefaultFrom = true, _b)
            }, minify && {
                minify: tslib.__assign(tslib.__assign({}, minifyOptions), {
                    sourceMap: options.sourcemap
                })
            }),
            sourceMaps: options.sourcemap,
            inlineSourcesContent: false
        }), 
    ].filter(function(n) {
        return Boolean(n);
    });
    return {
        input: entry,
        external: function external(id) {
            return externals.some(function(name) {
                return id === name || id.startsWith(name + "/");
            });
        },
        plugins: plugins,
        treeshake: {
            propertyReadSideEffects: false
        },
        onwarn: function onwarn(warning, warn) {
            var code = warning.code || "";
            if ([
                "MIXED_EXPORTS",
                "PREFER_NAMED_EXPORTS",
                "THIS_IS_UNDEFINED"
            ].includes(code)) return;
            if (code === "CIRCULAR_DEPENDENCY" && /Circular dependency: node_modules/.test(warning.message)) {
                return;
            }
            warn(warning);
        }
    };
}
function createOutputOptions(options, pkg, _a) {
    var tsCompilerOptions = _a.tsCompilerOptions;
    var format = options.format;
    var exportPaths = getExportPaths(pkg);
    var useEsModuleMark = Boolean(tsCompilerOptions.esModuleInterop || exportPaths.main && exportPaths.module);
    var file = path.resolve(options.file);
    return {
        name: pkg.name,
        file: file,
        format: format,
        exports: "named",
        esModule: useEsModuleMark && format !== "umd",
        freeze: false,
        strict: false,
        sourcemap: options.sourcemap
    };
}
function findExport(field) {
    if (!field) return;
    if (typeof field === "string") return field;
    var value = field["."] || field["import"] || field["module"] || field["default"];
    return findExport(value);
}
function parseExport(exportsCondition) {
    var paths = {};
    if (typeof exportsCondition === "string") {
        paths.export = exportsCondition;
    } else {
        paths.main = paths.main || exportsCondition["require"] || exportsCondition["node"] || exportsCondition["default"];
        paths.module = paths.module || exportsCondition["module"];
        paths.export = findExport(exportsCondition);
    }
    return paths;
}
function getExportPaths(pkg) {
    var pathsMap = {};
    var mainExport = {};
    if (pkg.main) {
        mainExport.main = pkg.main;
    }
    if (pkg.module) {
        mainExport.module = pkg.module;
    }
    pathsMap["."] = mainExport;
    var exportsConditions = pkg.exports;
    if (exportsConditions) {
        if (typeof exportsConditions === "string") {
            mainExport.export = exportsConditions;
        } else {
            var exportKeys = Object.keys(exportsConditions);
            if (exportKeys.some(function(key) {
                return key.startsWith(".");
            })) {
                exportKeys.forEach(function(subExport) {
                    pathsMap[subExport] = parseExport(exportsConditions[subExport]);
                });
            } else {
                Object.assign(mainExport, parseExport(exportsConditions));
            }
        }
    }
    pathsMap["."] = mainExport;
    return pathsMap;
}
function getExportDist(pkg) {
    var paths = getExportPaths(pkg)["."];
    var dist = [];
    if (paths.main) {
        dist.push({
            format: "cjs",
            file: getDistPath(paths.main)
        });
    }
    if (paths.module) {
        dist.push({
            format: "esm",
            file: getDistPath(paths.module)
        });
    }
    if (paths.export) {
        dist.push({
            format: "esm",
            file: getDistPath(paths.export)
        });
    }
    if (dist.length === 0) {
        dist.push({
            format: "esm",
            file: getDistPath("dist/index.js")
        });
    }
    return dist;
}
function getSubExportDist(pkg, exportCondition) {
    var dist = [];
    if (typeof exportCondition === "string") {
        dist.push({
            format: pkg.type === "module" ? "esm" : "cjs",
            file: getDistPath(exportCondition)
        });
    } else {
        var subExports = exportCondition;
        if (typeof subExports === "string") {
            dist.push({
                format: "esm",
                file: getDistPath(subExports)
            });
        } else {
            if (subExports.require) {
                dist.push({
                    format: "cjs",
                    file: getDistPath(subExports.require)
                });
            }
            if (subExports.import) {
                dist.push({
                    format: "esm",
                    file: getDistPath(subExports.import)
                });
            }
        }
    }
    return dist;
}
function createRollupConfig(entry, pkg, cliArgs) {
    var _a, _b;
    var file = cliArgs.file;
    var ext = path.extname(entry);
    var useTypescript = ext === ".ts" || ext === ".tsx";
    var options = tslib.__assign(tslib.__assign({}, cliArgs), {
        useTypescript: useTypescript
    });
    var tsCompilerOptions = {};
    var tsconfigPath;
    if (useTypescript) {
        var ts = resolveTypescript();
        tsconfigPath = path.resolve(config.rootDir, "tsconfig.json");
        if (fs__default["default"].existsSync(tsconfigPath)) {
            var tsconfigJSON = ts.readConfigFile(tsconfigPath, ts.sys.readFile).config;
            tsCompilerOptions = ts.parseJsonConfigFileContent(tsconfigJSON, ts.sys, "./").options;
        }
    }
    var typescriptOptions = {
        tsconfigPath: tsconfigPath,
        tsCompilerOptions: tsCompilerOptions
    };
    var inputOptions = createInputConfig(entry, pkg, options, typescriptOptions);
    var outputExports = options.exportCondition ? getSubExportDist(pkg, options.exportCondition.export) : getExportDist(pkg);
    var outputConfigs = outputExports.map(function(exportDist) {
        return createOutputOptions(tslib.__assign(tslib.__assign({}, cliArgs), {
            file: exportDist.file,
            format: exportDist.format,
            useTypescript: useTypescript
        }), pkg, typescriptOptions);
    });
    if (file) {
        var format = (_a = outputExports[0]) === null || _a === void 0 ? void 0 : _a.format;
        outputConfigs = [
            createOutputOptions(tslib.__assign(tslib.__assign({}, cliArgs), {
                file: file,
                format: cliArgs.format || format,
                useTypescript: useTypescript
            }), pkg, typescriptOptions), 
        ];
    }
    return {
        input: inputOptions,
        output: outputConfigs,
        exportName: ((_b = options.exportCondition) === null || _b === void 0 ? void 0 : _b.name) || "."
    };
}

function assignDefault(options, name, defaultValue) {
    if (!(name in options) || options[name] == null) {
        options[name] = defaultValue;
    }
}
function getSourcePathFromExportPath(cwd, exportPath) {
    var exts = [
        "js",
        "cjs",
        "mjs",
        "jsx",
        "ts",
        "tsx"
    ];
    for(var _i = 0, exts_1 = exts; _i < exts_1.length; _i++){
        var ext = exts_1[_i];
        if (exportPath.endsWith("package.json")) return;
        if (exportPath === ".") exportPath = "./index";
        var filename = path.resolve(cwd, "".concat(exportPath, ".").concat(ext));
        if (fs__default["default"].existsSync(filename)) {
            return filename;
        }
    }
    return;
}
function bundle(entryPath, _a) {
    if (_a === void 0) {
        _a = {};
    }
    var cwd = _a.cwd, options = tslib.__rest(_a, [
        "cwd"
    ]);
    config.rootDir = path.resolve(process.cwd(), cwd || "");
    assignDefault(options, "format", "es");
    assignDefault(options, "minify", false);
    assignDefault(options, "target", "es5");
    if (options.format === "esm") {
        options.format = "es";
    }
    var pkg = getPackageMeta();
    var packageExports = pkg.exports;
    var isSingleEntry = typeof packageExports === "string";
    var hasMultiEntries = packageExports && !isSingleEntry && Object.keys(packageExports).length > 0;
    var bundleOrWatch = function bundleOrWatch(rollupConfig) {
        if (options.watch) {
            return Promise.resolve(runWatch(pkg, rollupConfig));
        }
        return runBundle(pkg, rollupConfig);
    };
    if (isSingleEntry) {
        entryPath = getSourcePathFromExportPath(config.rootDir, ".");
    }
    if (!fs__default["default"].existsSync(entryPath)) {
        var hasEntryFile = entryPath === "" ? "" : fs__default["default"].statSync(entryPath).isFile();
        if (!hasEntryFile && !hasMultiEntries) {
            var err = new Error("Entry file is not existed");
            err.name = "NOT_EXISTED";
            return Promise.reject(err);
        }
        if (hasMultiEntries) {
            var rollupConfigs = Object.keys(packageExports).map(function(entryExport) {
                var source = getSourcePathFromExportPath(config.rootDir, entryExport);
                if (!source) return;
                options.exportCondition = {
                    source: source,
                    name: entryExport,
                    export: packageExports[entryExport]
                };
                var rollupConfig = createRollupConfig(path.resolve(cwd, source), pkg, options);
                return rollupConfig;
            }).filter(function(v) {
                return !!v;
            });
            return Promise.all(rollupConfigs.map(function(rollupConfig) {
                return bundleOrWatch(rollupConfig);
            }));
        }
    }
    var rollupConfig = createRollupConfig(entryPath, pkg, options);
    return bundleOrWatch(rollupConfig);
}
function getExportPath(pkg, exportName) {
    var name = pkg.name || path.basename(config.rootDir);
    if (exportName === "." || !exportName) return name;
    return path.join(name, exportName);
}
function runWatch(pkg, _a) {
    var exportName = _a.exportName, input = _a.input, output = _a.output;
    var watchOptions = [
        tslib.__assign(tslib.__assign({}, input), {
            output: output,
            watch: {
                exclude: [
                    "node_modules/**"
                ]
            }
        }), 
    ];
    var watcher = rollup.watch(watchOptions);
    var exportPath = getExportPath(pkg, exportName);
    var startTime = Date.now();
    watcher.on("event", function(event) {
        switch(event.code){
            case "ERROR":
                {
                    return onError(event.error);
                }
            case "START":
                {
                    startTime = Date.now();
                    logger.log("Start building ".concat(exportPath, " ..."));
                }
            case "END":
                {
                    var duration = Date.now() - startTime;
                    if (duration > 0) {
                        logger.log("✨ Built ".concat(exportPath));
                    }
                }
            default:
                return;
        }
    });
    return watcher;
}
function runBundle(pkg, _a) {
    var exportName = _a.exportName, input = _a.input, output = _a.output;
    var startTime = Date.now();
    return rollup.rollup(input).then(function(bundle) {
        var writeJobs = output.map(function(options) {
            return bundle.write(options);
        });
        return Promise.all(writeJobs);
    }, onError).then(function() {
        var duration = Date.now() - startTime;
        if (duration > 0) {
            logger.log("✨ Built ".concat(getExportPath(pkg, exportName)));
        }
    });
}
function onError(error) {
    if (!error) return;
    if (error.frame) {
        process.stderr.write(error.frame + "\n");
    }
    var err = new Error(error.message);
    err.stack = error.stack;
    throw err;
}

exports.bundle = bundle;
