import type { JscTarget } from '@swc/core';
import type { InputOptions, OutputOptions, RollupOptions } from 'rollup';
declare type ExportType = 'require' | 'export' | 'default' | string;
declare type CommonConfig = {
    format?: OutputOptions['format'];
    minify?: boolean;
    sourcemap?: boolean;
    external?: string[];
    runtime?: string;
    exportCondition?: {
        source: string;
        name: string;
        export: ExportCondition;
    };
};
declare type PackageMetadata = {
    name?: string;
    main?: string;
    module?: string;
    type?: 'commonjs' | 'module';
    dependencies?: Record<string, string>;
    peerDependencies?: Record<string, string>;
    peerDependenciesMeta?: Record<string, Record<string, string>>;
    exports?: string | Record<string, ExportCondition>;
    types?: string;
    typings?: string;
};
declare type ExportCondition = string | Record<ExportType, string>;
declare type BuncheeRollupConfig = Partial<Omit<RollupOptions, 'input' | 'output'>> & {
    exportName?: string;
    input: InputOptions;
    output: OutputOptions[];
};
declare type CliArgs = CommonConfig & {
    file?: string;
    watch?: boolean;
    cwd?: string;
    target?: JscTarget;
};
declare type BundleOptions = CliArgs & {
    useTypescript: boolean;
};
export type { CliArgs, ExportType, BundleOptions, ExportCondition, PackageMetadata, BuncheeRollupConfig };
